@@ -1,6 +1,6 @@
 package Data::FormValidator::Constraints::Business::DK::Postalcode;
 
-# $Id: Postalcode.pm 8057 2013-02-24 18:46:50Z jonasbn $
+# $Id$
 
 use strict;
 use warnings;
@@ -88,16 +88,16 @@ The documentation describes version 0.01 of Data::FormValidator::Constraints::Bu
 =head1 DESCRIPTION
 
 This module exposes a set of subroutines which are compatible with
-L<Data::FormValidator>. The module implements contraints as specified in
-L<Data::FormValidator::Constraints>.
+L<https://metacpan.org/pod/Data::FormValidator>. The module implements contraints as specified in
+L<https://metacpan.org/pod/Data::FormValidator::Constraints>.
 
-For a more through description of Danish postal codes please see: L<Business::DK::Postalcode>.
+For a more through description of Danish postal codes please see: L<https://metacpan.org/pod/Business::DK::Postalcode>.
 
 =head1 SUBROUTINES AND METHODS
 
 =head2 valid_postalcode
 
-Checks whether a Postalcode is valid (see: SYNOPSIS) and L<Business::DK::Postalcode>
+Checks whether a Postalcode is valid (see: SYNOPSIS) and L<https://metacpan.org/pod/Business::DK::Postalcode>
 
 =head2 match_postalcode
 
@@ -123,7 +123,7 @@ Data::FormValidator::Constraints::Business::DK::Postalcode exports on request:
 
 =over
 
-=item * Please refer to L<Data::FormValidator> for documentation on this
+=item * Please refer to L<https://metacpan.org/pod/Data::FormValidator> for documentation on this
 
 =back
 
@@ -135,9 +135,9 @@ The module requires no special configuration or environment to run.
 
 =over
 
-=item * L<Data::FormValidator>
+=item * L<https://metacpan.org/pod/Data::FormValidator>
 
-=item * L<Business::DK::Postalcode>
+=item * L<https://metacpan.org/pod/Business::DK::Postalcode>
 
 =back
 
@@ -148,7 +148,7 @@ The module has no known incompatibilities.
 =head1 BUGS AND LIMITATIONS
 
 The tests seem to reflect that untainting is taking place, but the L</match_postalcode>
-is not called at all, so how this untaiting is expected to be integrated into L<Data::FormValidator>
+is not called at all, so how this untaiting is expected to be integrated into L<https://metacpan.org/pod/Data::FormValidator>
 is still not clear to ne (SEE: TODO)
 
 =head1 TEST AND QUALITY
@@ -169,23 +169,23 @@ Coverage of the test suite is at 57.6%
 
 =over
 
-=item * L<Data::FormValidator>
+=item * L<https://metacpan.org/pod/Data::FormValidator>
 
-=item * L<Data::FormValidator::Constraints>
+=item * L<https://metacpan.org/pod/Data::FormValidator::Constraints>
 
-=item * L<Data::FormValidator::Result>
+=item * L<https://metacpan.org/pod/Data::FormValidator::Result>
 
-=item * L<Business::DK::Postalcode>
+=item * L<https://metacpan.org/pod/Business::DK::Postalcode>
 
-=item * L<Data::FormValidator::Constraints::Business::DK::CVR>
+=item * L<https://metacpan.org/pod/Data::FormValidator::Constraints::Business::DK::CVR>
 
-=item * L<Data::FormValidator::Constraints::Business::DK::CPR>
+=item * L<https://metacpan.org/pod/Data::FormValidator::Constraints::Business::DK::CPR>
 
-=item * L<Data::FormValidator::Constraints::Business::DK::FI>
+=item * L<https://metacpan.org/pod/Data::FormValidator::Constraints::Business::DK::FI>
 
-=item * L<Data::FormValidator::Constraints::Business::DK::PO>
+=item * L<https://metacpan.org/pod/Data::FormValidator::Constraints::Business::DK::PO>
 
-=item * L<Data::FormValidator::Constraints::Business::DK::Phonenumber>
+=item * L<https://metacpan.org/pod/Data::FormValidator::Constraints::Business::DK::Phonenumber>
 
 =back
 
@@ -198,7 +198,7 @@ Please report issues via CPAN RT:
 or by sending mail to
 
   bug-Business-DK-Postalcode@rt.cpan.org
-  
+
 =head1 AUTHOR
 
 Jonas B. Nielsen, (jonasbn) - C<< <jonasbn@cpan.org> >>
